<?php
new MaroyakaWebClapAjax();

class MaroyakaWebClapAjax extends MaroyakaWebClap {

	function __construct() {
		parent::__construct();
		add_action('wp_ajax_claping', array($this, 'claping'));
		add_action('wp_ajax_nopriv_claping', array($this, 'claping'));
		add_action('wp_ajax_send_message', array($this, 'send_message'));
		add_action('wp_ajax_nopriv_send_message', array($this, 'send_message'));
		add_action('wp_ajax_ranking', array($this, 'ranking'));
		add_action('wp_ajax_nopriv_ranking', array($this, 'ranking'));
	}

	///////////////////////////////////////
	/* 拍手処理 */
	///////////////////////////////////////
	public function claping() {
		$post_id = esc_attr($this->get_post('post_id'));

		check_ajax_referer('mwc_claping');

		$usip = getenv('REMOTE_ADDR');

		$errors = array();

		// アクセス制限
		foreach ($this->get_option('deny_ips') as $ip) {
			if ($ip === '') continue;
			$denyip = preg_replace("/\./", "\\.", $ip);
			$denyip = preg_replace("/\*/", ".*", $denyip);
			if (preg_match("/$denyip/i", $usip)) {
				$this->json_output(array('error' => 'アクセスが許可されていません。'));
			}
		}

		// 連続拍手数限度
		$max_claping = $this->get_option('max_claping');
		if ($max_claping > 0) {
			$reclap_time = date_i18n("Y-m-d H:i:s", date_i18n("U") - (60 * 60 * $this->get_option('reclap_time')));
			$usclap_count = $this->db->get_var($this->db->prepare("SELECT count(*) FROM ". $this->prefix. MWC_DB_TABLE_CLAP. " WHERE post_id = %d AND ip = %s AND timestamp >= %s", $post_id, $usip, $reclap_time));
			if ($max_claping > 0 && $usclap_count >= $max_claping) {
				$this->json_output(array('error' => '連続拍手限度数を超えています。'));
			}
		}

		// 拍手データ追加
		$clap_ins_result = $this->db->insert($this->prefix. MWC_DB_TABLE_CLAP,
			array(
				'post_id' => $post_id,
				'ip' => $usip,
			),
			array(
				'%d',
				'%s',
			)
		);
		$clap_id = $this->db->insert_id;

		// 記事タイトルをラベルにする
		$post_title = get_the_title($post_id);

		// 個別拍手数
		$clap_count = $this->db->get_var($this->db->prepare("SELECT * FROM ". $this->prefix. MWC_DB_TABLE_ID. " WHERE post_id = %d", $post_id), 2);

		// 拍手数加算
		if (is_null($clap_count)) {
			$count = 1;
			$id_ins_result = $this->db->insert($this->prefix. MWC_DB_TABLE_ID,
				array(
					'post_id' => $post_id,
					'count' => 1,
					'label' => $post_title,
				),
				array(
					'%d',
					'%d',
					'%s',
				)
			);
		} else {
			$count = $clap_count + 1;
			$id_up_result = $this->db->update($this->prefix. MWC_DB_TABLE_ID,
				array(
					'count' => $count,
					'label' => $post_title,
				),
				array('post_id' => $post_id),
				array(
					'%d',
					'%s',
				),
				array('%d')
			);
		}

		// 結果表示
		$result = array(
			'thanks_msg' => nl2br($this->get_option('thanks_msg')),
			'name_input' => $this->get_option('name_input'),
			'set_phrases_select' => $this->get_option('set_phrases_select'),
			'message_input' => $this->get_option('message_input'),
			'set_phrases' => $this->get_option('set_phrases'),
			'post_id' => $post_id,
			'clap_id' => $clap_id,
			'count' => $count,
			'close_img' => MWC_PLUGIN_URL. '/img/close.png',
			'ajax_nonce' => wp_create_nonce('mwc_send_message_'. $post_id),
		);

		$this->json_output($result);
	}

	///////////////////////////////////////
	/* メッセージ送信処理 */
	///////////////////////////////////////
	public function send_message() {
		$post_id = esc_attr($this->get_post('post_id'));

		check_ajax_referer('mwc_send_message_'. $post_id);

		$name = esc_attr($this->get_post('name'));
		$set_phrases = esc_attr($this->get_post('set_phrases'));
		$message = esc_attr($this->get_post('message'));

		// メッセージ保存
		$msg_ins_result = $this->db->insert($this->prefix. MWC_DB_TABLE_MESSAGES,
			array(
				'clap_id' => esc_attr($this->get_post('clap_id')),
				'name' => $name,
				'set_phrases' => $set_phrases,
				'message' => $message,
			),
			array(
				'%d',
				'%s',
				'%s',
				'%s',
			)
		);

		// メール通知
		if (
			($this->get_option('mail_notice_name') == 1 && !empty($name)) or
			($this->get_option('mail_notice_sp') == 1 && !empty($set_phrases)) or
			($this->get_option('mail_notice_msg') == 1 && !empty($message))
			) {
			$mail = $this->get_option('mailaddress');
			$to = (!empty($mail)) ? $mail : get_option('admin_email');
			$subject = 'WEB拍手通知';
			$body = "下記の内容で拍手がありました。\n".
								"--------------------------------------------------------\n".
								"【名前】". $name. "\n".
								"【定型文】". $set_phrases. "\n".
								"【メッセージ】". $message. "\n".
								"--------------------------------------------------------\n".
								"日付：". date_i18n("Y-m-d H:i:s"). "\n";
			wp_mail($mail, $subject, $body);
		}

		// 結果表示
		$this->json_output(array(1));
	}

	///////////////////////////////////////
	/* ランキング */
	///////////////////////////////////////
	public function ranking() {
		$widgetID = esc_attr($this->get_post('widgetID'));
		$maxview = esc_attr($this->get_post('maxview', 0));
		$mode = esc_attr($this->get_post('mode'));

		$wop = get_option('widget_MaroyakaWebClapWidget');
		$op = $wop[$widgetID];

		$where = " WHERE ". $this->db->posts. ".post_status = 'publish'".
								" AND IFNULL(". $this->db->postmeta. ".meta_value, 0) != '1'";
		if (in_array($mode, array('today', 'yesterday', 'week', 'month'))) {
			$sdate = date_i18n("Y-m-d", strtotime(date_i18n("Y-m-d"). ' -1 '. $mode));
			$where .= " AND clp.timestamp >= '". $sdate. " 00:00:00'";
		}

		$sql = "SELECT count(*) as count, clp.post_id, cid.label".
						" FROM ". $this->prefix. MWC_DB_TABLE_CLAP. " AS clp".
						" LEFT JOIN ". $this->prefix. MWC_DB_TABLE_ID. " AS cid ON clp.post_id = cid.post_id".
						" LEFT JOIN ". $this->db->posts. " ON clp.post_id = ". $this->db->posts. ".ID".
						" LEFT JOIN ". $this->db->postmeta.
							" ON clp.post_id = ". $this->db->postmeta. ".post_id".
								" AND ". $this->db->postmeta. ".meta_key = 'mwc_deny_btn'".
						$where.
						" GROUP BY clp.post_id".
						" ORDER BY count DESC, cid.post_id ASC".
						" LIMIT ". $maxview;
		$clap_list = $this->db->get_results($sql);

		$rows = '';
		$rank = 0; $prev = -1; $i = 0; $last = count($clap_list);
		foreach ($clap_list as $row) {
			$i++;
			if ($prev !== $row->count) $rank = $i;
			$prev = $row->count;
			$label = esc_attr($row->label);
			if ($op['view_id'] == 1) {
				$label = '('. $row->post_id. ') '. $label;
			}
			$rows .= '<div class="tr">'.
				'<div class="td column-rank">'. $rank. '</div>'.
				'<div class="td column-count">'. $row->count. '</div>'.
				'<div class="td column-label">'.
					'<a href="'. get_permalink(esc_attr($row->post_id)). '" target="_blank" title="'. $label. '">'.
						$label.
						(($op['view_eyecatch'] == 1) ?
							(!empty($label) ? '<br>' : '').
							get_the_post_thumbnail($row->post_id, array(50, 50))
						: '').
					'</a>'.
				'</div>'.
			'</div>';
		}

		$this->json_output($rows);
	}

	///////////////////////////////////////
	/* JSON出力 */
	///////////////////////////////////////
	private function json_output($result) {
		nocache_headers();
		header('Content-Type: application/json; charset='. get_bloginfo('charset'));
		echo json_encode($result);
		die();
	}
}
