<?php
class MWC_Clap_List_Table extends WP_List_Table {

	public $query;
	public $total_item_count;
	public $view_eyecatch = false;
	public $view_post_id = false;
	public $hidden_columns = array();

	/**
	 * Constructor, we override the parent to pass our own arguments
	 * We usually focus on three parameters: singular and plural labels, as well as whether the class supports AJAX.
	 */
	function __construct() {
		parent::__construct(array(
			'singular'=> 'wp_list_text_link', //Singular label
			'plural' => 'wp_list_test_links', //plural label, also this well be one of the table css class
			'ajax'	=> false //We won't support Ajax for this table
		));
	}

	/**
	 * Define the columns that are going to be used in the table
	 * @return array $columns, the array of columns to use with the table
	 */
	public function get_columns() {
		return $columns= array(
			'timestamp'=> '日時',
			'label'=> 'ページ',
			'clap'=>'拍手数',
			'ip'=>'IP',
			'name'=>'名前',
			'set_phrases'=>'定型文',
			'message'=>'メッセージ',
		);
	}

	function column_timestamp($item) {
		$week = array('日', '月', '火', '水', '木', '金', '土');
		$time = strtotime($item->timestamp);
		$tt = date_i18n('U') - $time;

		$date = date_i18n('Y年m月d日('. $week[date('w', $time)]. ') H:i', $time);

		if ($tt >= 0 && $tt < 10) {
			$ts = sprintf("今");
		} elseif ($tt >= 10 && $tt < 60) {
			$ts = sprintf("%d秒前", $tt);
		} elseif ($tt >= 60 && $tt < 3600) {
			$ts = sprintf("%d分前", ($tt / 60));
		} elseif ($tt >= 3600 && $tt < 86400 ) {
			$ts = sprintf("%d時間前", ($tt / 60 / 60));
		// } elseif ($tt >= 86400 && $tt < 2592000 ) {
		// 	$ts = sprintf("%d日前", ($tt / 60 / 60 / 24));
		} else {
			$ts = $date;
		}

		$result = '<abbr title="'. $date. '">'. $ts. '</abbr>';

		return $result;
	}

	function column_clap($item) {
		return '<span>'. $item->clap. '</span>';
	}

	function column_ip($item) {
		$output = "\n" . '<input type="text"'
			.' onfocus="this.select();" readonly="readonly"'
			.' value="' . esc_attr( $item->ip ) . '"'
			.' class="ip-in-list-table wp-ui-text-highlight code" />';

		return trim($output);
	}

	/**
	* Define which columns are hidden
	*
	* @return Array
	*/
	public function get_hidden_columns() {
		return $this->hidden_columns;
	}

	public function add_hidden_columns($col) {
		array_push($this->hidden_columns, $col);
	}

	/**
	 * Decide which columns to activate the sorting functionality on
	 * @return array $sortable, the array of columns that can be sorted by the user
	 */
	public function get_sortable_columns() {
		return $sortable = array(
			'timestamp' => array('timestamp', true),
			'label' => array('label', true),
			'clap' => array('clap', true),
			'ip' => array('ip', true),
			'name' => array('name', true),
			'set_phrases' => array('set_phrases', true),
			'message' => array('message', true),
		);
	}

	public function column_default($item, $column_name){
		return $item->$column_name;
	}

	public function column_label($item) {
		$output = '<a href="'. get_permalink($item->post_id). '" target="_blank">';
		if ($this->view_post_id) {
			$item->label = '('. $item->post_id. ') '. $item->label;
		}
		$output .= esc_attr($item->label);
		if ($this->view_eyecatch) {
			if ($this->view_post_id) {
				$output .= '<br>';
			}
			$output .= get_the_post_thumbnail($item->post_id, array(150, 150));
		}
		$output .= '</a>';
		return $output;
	}

	// Used to display the value of the id column
	// public function column_id($item)
	// {
	// 	return $item['id'];
	// }

	/**
	* Allows you to sort the data by the variables set in the $_GET
	*
	* @return Mixed
	*/
	// private function sort_data($a, $b){
	// 	// Set defaults
	// 	$orderby = 'title';
	// 	$order = 'asc';

	// 	// If orderby is set, use this as the sort column
	// 	if(!empty($_GET['orderby'])) {
	// 		$orderby = $_GET['orderby'];
	// 	}

	// 	// If order is set use this as the order
	// 	if(!empty($_GET['order'])) {
	// 		$order = $_GET['order'];
	// 	}

	// 	$result = strcmp( $a[$orderby], $b[$orderby] );

	// 	if($order === 'asc') {
	// 		return $result;
	// 	}

	// 	return -$result;
	// }

	/**
	 * Prepare the table with different parameters, pagination, columns and table elements
	 */
	public function prepare_items() {
		global $wpdb;

		$columns = $this->get_columns();
		$hidden = $this->get_hidden_columns();
		$sortable = $this->get_sortable_columns();

		/* -- Ordering parameters -- */
		$query = $this->query;
		$orderby = (isset($_GET["orderby"]) && in_array($_GET["orderby"], array_keys($columns))) ? $_GET["orderby"] : 'timestamp';
		$order = (isset($_GET["order"]) && in_array($_GET["order"], array('asc', 'desc'))) ? strtoupper($_GET["order"]) : 'DESC';
		$query .=' ORDER BY '. $orderby. ' '. $order.', ms.id DESC';

		/* -- Pagination parameters -- */
		$data = $wpdb->get_results($query); //return the total number of affected rows

		// usort($data, array(&$this, 'sort_data'));

		$perpage = 20;
		$currentpage = $this->get_pagenum();
		$total = count($data);
		$this->total_item_count = $total;

		$this->set_pagination_args(array(
			'total_items' => $total,
			'per_page'    => $perpage
		));

		$totalitems = array_slice($data, (($currentpage-1) * $perpage), $perpage);

		$this->_column_headers = array($columns, $hidden, $sortable);
		$this->items = $totalitems;
	}

	public function total_items() {
		return $this->total_item_count;
	}

	public function total_unique_items() {
		$uniques = array();
		foreach ($this->items as $item)
			if (!isset($uniques[$item->ip])) $uniques[$item->ip] = 1;
		return count($uniques);
	}
}
